package core;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import jpcap.JpcapCaptor;
import jpcap.NetworkInterface;
import jpcap.PacketReceiver;
import jpcap.packet.Packet;
import packetfilter.ICMPFilter;
import packetfilter.PacketFilter;
import packetfilter.TCPFilter;
import packetfilter.UDPFilter;

/*
 * Class que calcula o tamanho mdio dos pacotes
 * 
 * e tambm o nmero de pacotes por segundo
 * 
 */
public class PacketMain {

	public final static int TCP = 0, UDP = 1, ICMP = 2, OTHER = 3;
	private List<List<Packet>> ps;
	private long[] sizeOfPackets;
	private long[] numberOfPackets;
	private List<PacketFilter> filters = new ArrayList<PacketFilter>();
	private final static int MAX_SIZE = 2000;
	private Date[] first;
	private Date[] last;



	private PacketReceiver handler=new PacketReceiver(){

		public void receivePacket(Packet packet) {
			int index = -1;

			for(int i = 0; i < filters.size(); i++)
				if(filters.get(i).isFiltered(packet)) {
					index = i;
					break;
				}
			if(index == -1)
				index = sizeOfPackets.length - 1;


			while(ps.get(index).size() > MAX_SIZE) {
				sizeOfPackets[index] -= ps.get(index).remove(0).len;
				numberOfPackets[index]--;
			}
			ps.get(index).add(packet);
			sizeOfPackets[index]+= packet.len;
			numberOfPackets[index]++;
			Packet fp, lp;
			fp = ps.get(index).get(0);
			lp=packet;

			first[index]=new Date(fp.sec*1000+fp.usec/1000);
			last[index]=new Date(lp.sec*1000+lp.usec/1000);


		}
	};

	public PacketMain() {
		this.ps = new ArrayList<List<Packet>>();
		ps.add(new ArrayList<Packet>());
		ps.add(new ArrayList<Packet>());
		ps.add(new ArrayList<Packet>());
		ps.add(new ArrayList<Packet>());
		
		filters.add(new TCPFilter());
		filters.add(new UDPFilter());
		filters.add(new ICMPFilter());

		sizeOfPackets = new long[filters.size()+1];
		numberOfPackets = new long[filters.size()+1];
		first = new Date[filters.size()+1];
		last = new Date[filters.size()+1];

	}



	public double getAveragePacketSize(int index)
	{
		long secondL;

		if(last[index] == null)
			secondL = 0l;
		else
			secondL = last[index].getTime();

		double sec = ((double)(System.currentTimeMillis() - secondL)/ 1000d);
		Double value = ((double)sizeOfPackets[index] / numberOfPackets[index])/(sec*50);
		if(Double.isNaN(value) || Double.isInfinite(value) || value <= 0)
			return 1;
		if(value > 20000)
			return 20000;
		return value;
	}

	public void start(NetworkInterface device) throws IOException {

		final JpcapCaptor jpcap = JpcapCaptor.openDevice(device, 1514, true, 50);

		Thread process = new Thread(
				new Runnable() {
					public void run() {
						while(true) {
							jpcap.processPacket(1, handler);

						}
					}
				});

		process.start();
	}

	public double getNumberOfPacketsPerSecond(int index) {
		
		long secondL;

		if(last[index] == null)
			secondL = 0l;
		else
			secondL = last[index].getTime();

		double sec = ((double)(System.currentTimeMillis() - secondL)/ 1000d);

		double value = ((double)numberOfPackets[index] /  sec) / 200;
		//System.out.println(numberOfPackets[index]);
		if(Double.isInfinite(value) || Double.isNaN(value) || value <= 0)
			value = 1d;
		if(value > 20000)
			value = 20000;
		return value;
	}
	
}





