package ui;
import instruments.MeuInstrRT;
import instruments.SineInstRT;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import jm.JMC;
import jm.audio.Instrument;
import jm.audio.RTMixer;
import jm.audio.synth.Oscillator;
import jm.music.rt.RTLine;
import jpcap.JpcapCaptor;
import jpcap.NetworkInterface;
import melodies.RealtimeMelody;
import melodies.ThreadMelody;
import core.PacketMain;

/* Tela Inicial */
public class MainFrame extends JFrame implements ActionListener, JMC 
{
	
	public final static int SINE = 0, SAWDOWN = 1, TRIANGLE = 2, SABERSAW = 3;
	private static final long serialVersionUID = 8707918700206249889L;
	private JLabel packets;	
	private JButton startButton,stopButton;
	private JComboBox comboBoxDevice, comboBoxIntrument[], comboBoxFunction[], comboBoxProtocol[];
	private String[] instrumentNames = {"SINE_WAVE","SAWDOWN_WAVE","TRIANGLE_WAVE","SABERSAW_WAVE"};
	private String[] functionNames = {"Tamanho Mdio de Pacotes","Nmero de Pacotes/s"};
	private String[] function = {"getAveragePacketSize","getNumberOfPacketsPerSecond"};
	private String[] protocolNames = {"TCP","UDP","ICMP","OTHER"};
	private NetworkInterface[] devices;
	private PacketMain p;
	private Instrument[] instruments;
	private int sampleRate = 44100;
	private RTMixer rtm;
	private ThreadMelody melody1;
	private ThreadMelody melody2;
	private ThreadMelody melody3;
	private ThreadMelody melody4;
	private ThreadMelody melodies[] = new ThreadMelody[4];
	

	public MainFrame(){
		
		setTitle("Main Window");

		JPanel panel = new JPanel(new BorderLayout());
		JPanel panel1 = new JPanel(new FlowLayout());

		startButton = new JButton("Start");
		startButton.setActionCommand("Start");
		startButton.addActionListener(this);

		panel1.add(startButton);

		stopButton = new JButton("Stop");
		stopButton.setActionCommand("Stop");
		stopButton.addActionListener(this);

		panel1.add(stopButton);

		panel.add(panel1, BorderLayout.SOUTH);

		packets = new JLabel("Started.");

		devices = JpcapCaptor.getDeviceList();

		if(devices == null) {

			JOptionPane.showMessageDialog(this,"No device found.");
			return;

		}

		else {

			String[] names = new String[devices.length];

			for(int i = 0 ; i < names.length ; i++)
				names[i] = (devices[i].description == null ? devices[i].name : devices[i].description);
			comboBoxDevice = new JComboBox(names);

		}

		panel.add(comboBoxDevice,BorderLayout.NORTH);

		JPanel main = new JPanel(new GridLayout(1,3));

		JLabel labelInstrument = new JLabel("Selecione Instrumento");
		
		JPanel panelInstrument = new JPanel(new FlowLayout());
		comboBoxIntrument = new JComboBox[4];

		JLabel labelFunction = new JLabel("Selecione Mtodo");
		
		JPanel panelFunction = new JPanel(new FlowLayout());
		comboBoxFunction = new JComboBox[4];

		JLabel labelProtocol = new JLabel("Selecione Protocolo           ");
		
		JPanel panelProtocol = new JPanel(new FlowLayout());
		comboBoxProtocol = new JComboBox[4];

		panelInstrument.add(labelInstrument);
		for(int i = 0; i < 4; i++) {
			comboBoxIntrument[i] = new JComboBox(instrumentNames);
			panelInstrument.add(comboBoxIntrument[i]);
		}
		main.add(panelInstrument,BorderLayout.WEST);
		
		panelFunction.add(labelFunction);
		for(int i = 0; i < 4; i++) {
			comboBoxFunction[i] = new JComboBox(functionNames);
			panelFunction.add(comboBoxFunction[i]);
		}
		main.add(panelFunction,BorderLayout.CENTER);
		
		panelProtocol.add(labelProtocol);
		for(int i = 0; i < 4; i++) {
			comboBoxProtocol[i] = new JComboBox(protocolNames);
			panelProtocol.add(comboBoxProtocol[i]);
		}
		main.add(panelProtocol,BorderLayout.EAST);
		
		panel.add(main);
		
		setSize(640, 480);
		setLocation(0, 0);
		
		getContentPane().setLayout(new BorderLayout());

		getContentPane().add(panel,BorderLayout.CENTER);
		
		getContentPane().add(packets,BorderLayout.SOUTH);

	}
	
	
	public void openWindow(PacketMain p) {
		this.p = p;
		this.setVisible(true);
	}
	
	public void actionPerformed(ActionEvent evt){
		String cmd=evt.getActionCommand();
		
		if(cmd.equals("Start")){
			
			if((melody1 != null && melody1.isAlive()) || (melody2 != null && melody2.isAlive()) || (melody3 != null && melody3.isAlive()) || (melody4 != null && melody4.isAlive()))
				return;
			
			try {
				p.start(devices[comboBoxDevice.getSelectedIndex() != -1 ? comboBoxDevice.getSelectedIndex() : 1]);
				instruments = new Instrument[4];

				instruments[SINE] = new SineInstRT(sampleRate);
				instruments[SAWDOWN] = new MeuInstrRT(Oscillator.SAWDOWN_WAVE);
				instruments[TRIANGLE] = new MeuInstrRT(Oscillator.TRIANGLE_WAVE);
				instruments[SABERSAW] = new MeuInstrRT(Oscillator.SABERSAW_WAVE);
				
				instruments[SINE].setOutput(1);
				instruments[SAWDOWN].setOutput(1);
						
				RealtimeMelody averageSize = new RealtimeMelody(new Instrument[]{instruments[comboBoxIntrument[0].getSelectedIndex() != -1 ? comboBoxIntrument[0].getSelectedIndex() : 0]});
				RealtimeMelody frequencySize = new RealtimeMelody(new Instrument[]{instruments[comboBoxIntrument[1].getSelectedIndex() != -1 ? comboBoxIntrument[1].getSelectedIndex() : 1]});
				RealtimeMelody testeSize = new RealtimeMelody(new Instrument[]{instruments[comboBoxIntrument[2].getSelectedIndex() != -1 ? comboBoxIntrument[2].getSelectedIndex() : 2]});
				RealtimeMelody testeSize2 = new RealtimeMelody(new Instrument[]{instruments[comboBoxIntrument[3].getSelectedIndex() != -1 ? comboBoxIntrument[3].getSelectedIndex() : 3]});
				
				RTLine[] rtlines = {averageSize,frequencySize,testeSize,testeSize2};
				rtm = new RTMixer(rtlines);

				averageSize.setDuration(0.5d);
				frequencySize.setDuration(0.9d);
				testeSize.setDuration(1.3d);
				testeSize2.setDuration(0.7d);

				melody1 = new ThreadMelody(averageSize,1000,p,function[comboBoxFunction[0].getSelectedIndex() != -1 ? comboBoxFunction[0].getSelectedIndex() : 0],comboBoxProtocol[0].getSelectedIndex() != -1 ? comboBoxProtocol[0].getSelectedIndex() : 0);
				melody2 = new ThreadMelody(frequencySize,1000,p,function[comboBoxFunction[1].getSelectedIndex() != -1 ? comboBoxFunction[1].getSelectedIndex() : 1],comboBoxProtocol[1].getSelectedIndex() != -1 ? comboBoxProtocol[1].getSelectedIndex() : 0);
				melody3 = new ThreadMelody(testeSize,3000,p,function[comboBoxFunction[2].getSelectedIndex() != -1 ? comboBoxFunction[2].getSelectedIndex() : 0],comboBoxProtocol[2].getSelectedIndex() != -1 ? comboBoxProtocol[2].getSelectedIndex() : 0);
				melody4 = new ThreadMelody(testeSize2,3000,p,function[comboBoxFunction[3].getSelectedIndex() != -1 ? comboBoxFunction[3].getSelectedIndex() : 1],comboBoxProtocol[3].getSelectedIndex() != -1 ? comboBoxProtocol[3].getSelectedIndex() : 0);
				
				melodies[0] = melody1;
				melodies[1] = melody2;
				melodies[2] = melody3;
				melodies[3] = melody4;
				
				FrameAux.startAll(rtm, melodies);
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}else if(cmd.equals("Stop")){
			
			rtm.finalize();
			melody1.interrupt();
			melody1 = null;
			melody2.interrupt();
			melody2 = null;
			melody3.interrupt();
			melody3 = null;
			melody4.interrupt();
			melody4 = null;
			
			
		}
	}

	
}
